/**
 * InfoCommand - a command to show Solaris information
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.solview;

import java.io.File;
import uk.co.petertribble.solview.helpers.RunCommand;

public class InfoCommand {

  private String text;
  private String cmd;
  private String args;
  private String fullcmd;

  /**
   * An informational command.
   */
  public InfoCommand(String text, String cmd) {
    this(text, cmd, (String) null);
  }

  /**
   * An informational command. with arguments.
   */
  public InfoCommand(String text, String cmd, String args) {
    this.text = text;
    this.cmd = cmd;
    this.args = args;
    if (args == null) {
      fullcmd = cmd;
    } else {
      fullcmd = cmd + " " + args;
    }
  }

  /**
   * Override toString() to give the informational
   * name.
   */
  public String toString() {
    return text;
  }

  /**
   * Retunr the full comand executed for this informational command, including
   * any arguments.
   */
  public String getFullCmd() {
    return fullcmd;
  }

  /**
   * Return whether the filename corresponding to the pathname exists.
   */
  public boolean exists() {
    File f = new File(cmd);
    return f.exists();
  }

  /**
   * Return the textual output from executing this informational command.
   */
  public String getOutput() {
    if (!exists()) {
      return "Command not found";
    }
    RunCommand rc = new RunCommand(fullcmd);
    return rc.getOut();
  }

}
