/**
 * ClusterPanel
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.solview;

import javax.swing.*;
import javax.swing.tree.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import uk.co.petertribble.jumble.*;
import java.io.File;

public class InstalledClusterPanel extends JPanel {

  //Like ClusterPanel, but for the currently installed system

  JEditorPane tp_info;
  JEditorPane tp_dep;
  JEditorPane tp_rdep;
  JEditorPane tp_cl;
  PkgList plist;
  ClusterToc ctoc;

  JTabbedPane jtp_l;
  JTabbedPane jtp_r;

  String currentPackage = null;

  public InstalledClusterPanel() {
    setLayout(new BorderLayout());

    //tabs for different views
    jtp_l = new JTabbedPane();

    plist = new PkgList();
    plist.addMouseListener(mouseListener);
    jtp_l.add("Packages",new JScrollPane(plist));

    //Which cluster do we have installed?
    Hashtable hclust = JumbleUtils.stringToPropHash(JumbleFile.getStringContents(new File("/var/sadm/system/admin/CLUSTER")),"\n");
    String instcluster = (String) hclust.get("CLUSTER");

    //tree view of clusters
    String toptext = instcluster;
    if (instcluster == null) {
      toptext = "Solaris";
    }
    DefaultMutableTreeNode topnode = new DefaultMutableTreeNode(toptext);
    final JTree tree = new JTree(topnode);
    ctoc = new ClusterToc(new java.io.File("/var/sadm/system/admin/.clustertoc"));
    if (instcluster == null) {
      ClusterTree ct = new ClusterTree(topnode, ctoc);
    } else {
      ClusterTree ct = new ClusterTree(topnode, ctoc, instcluster);
    }
    tree.expandRow(0);
    //Listen for when the selection changes.
    tree.addTreeSelectionListener(new TreeSelectionListener() {
      public void valueChanged(TreeSelectionEvent e) {
	DefaultMutableTreeNode node = (DefaultMutableTreeNode)
	  tree.getLastSelectedPathComponent();
	
	if (node == null) return;
	
	if (node.isLeaf()) {
	  setPackageInfo(node.getUserObject().toString());
	} else {
	  setClusterInfo(node.getUserObject().toString());
	}
      }
    });
    // end of tree listener
    jtp_l.add("Clusters", new JScrollPane(tree));

    //tabbed info pane on right
    jtp_r = new JTabbedPane();
    tp_info = new JEditorPane();
    tp_info.setContentType("text/html");
    tp_dep = new JEditorPane();
    tp_dep.setContentType("text/html");
    tp_rdep = new JEditorPane();
    tp_rdep.setContentType("text/html");
    tp_cl = new JEditorPane();
    tp_cl.setContentType("text/html");
    jtp_r.add("Info", new JScrollPane(tp_info));
    jtp_r.add("Dependencies", new JScrollPane(tp_dep));
    jtp_r.add("Clusters", new JScrollPane(tp_cl));

    //split pane to hold the lot
    JSplitPane psplit = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,jtp_l,jtp_r);
    psplit.setOneTouchExpandable(true);
    psplit.setDividerLocation(180);
    add(psplit);
  }

  MouseListener mouseListener = new MouseAdapter() {
    public void mouseClicked(MouseEvent e) {
      Cursor c = getCursor();
      setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
      JList source = (JList) e.getSource();
      setPackageInfo((String) source.getSelectedValue());
      setCursor(c);
    }
  };

  private void setClusterInfo(String s) {
    currentPackage = null;
    PackageCluster pc = ctoc.getMetaCluster(s);
    if (pc == null) {
      pc = ctoc.getCluster(s);
    }
    if (pc == null) {
      setInfoText(s);
    } else {
      StringBuffer sb = new StringBuffer();
      sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\"><th>Cluster Name</th></tr>\n");
      sb.append("<tr><td>").append(pc.getName()).append("</td></tr>\n");
      sb.append("<tr bgcolor=\"#eeeeee\"><th>Installed Status</th></tr>\n");
      sb.append("<tr><td>").append(clusterStatus(pc)).append("</td></tr>\n");
      sb.append("</table>\n");
      setInfoText(sb.toString());
    }
  }

  private void setClusterMembership(String pname) {
    StringBuffer sb = new StringBuffer();
    sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\"><th colspan=\"2\">This package is part of the following clusters:</th></tr>\n");
    Hashtable h = ctoc.getContainingClusters(pname);
    for (Enumeration e = h.keys(); e.hasMoreElements() ; ) {
      String cname = (String) e.nextElement();
      PackageCluster pclust = (PackageCluster) h.get(cname);
      sb.append("<tr><td>").append(cname).append("</td><td>").append(pclust.getName()).append("</td></tr>\n");
    }
    sb.append("</table>\n");
    tp_cl.setText(sb.toString());
    tp_cl.setMargin(new Insets(5,5,5,5));
    tp_cl.setCaretPosition(0);
    tp_cl.setEditable(false);
  }

  private String clusterStatus(PackageCluster pc) {
    int ipkg = 0;
    int iinst = 0;
    for (Enumeration e = pc.getPackages().elements(); e.hasMoreElements() ; ) {
      String pname = (String) e.nextElement();
      ipkg++;
      if (plist.exists(pname)) {
	iinst++;
      }
    }
    if (iinst == 0) {
      return "Not Installed";
    }
    if (iinst == ipkg) {
      return "Fully Installed";
    }
    return "Partially Installed";
  }

  private void setPackageInfo(String p) {
    if (plist.exists(p)) {
      currentPackage = p;
      setInfoText(plist.infoTable(p), plist.dependencyTable(p), plist.revDeps(p));
    } else {
      setInfoText("Not installed");
    }
    setClusterMembership(p);
  }

  private void setInfoText(String si) {
    setInfoText(si, "", "");
  }

  private void setInfoText(String si, String sd, String srd) {
    tp_info.setText(si);
    tp_info.setMargin(new Insets(5,5,5,5));
    tp_info.setCaretPosition(0);
    tp_info.setEditable(false);
    tp_dep.setText(sd);
    tp_dep.setMargin(new Insets(5,5,5,5));
    tp_dep.setCaretPosition(0);
    tp_dep.setEditable(false);
    tp_rdep.setText(srd);
    tp_rdep.setMargin(new Insets(5,5,5,5));
    tp_rdep.setCaretPosition(0);
    tp_rdep.setEditable(false);
  }

  public void createDependencies() {
    plist.createDependencies();
  }

  public void createRevDependencies() {
    plist.createRevDependencies();
    jtp_r.add("Dependants", new JScrollPane(tp_rdep));
    if (currentPackage != null) {
      setPackageInfo(currentPackage);
    }
  }

}
