/**
 * PkgList - shows available Solaris packages in a JList
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.solview;

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class PkgList extends JList {

  private PkgUtils pkgutil;
  private Hashtable Dependencies = null;
  private Hashtable revDependencies = null;

  DefaultListModel model;

  /**
   * Display a package list
   */
  public PkgList() {
    this("/var/sadm/pkg");
  }

  /**
   * Display a package list
   *
   * Packages are in directories, so ignore files
   * and anything hidden (starting with a dot) and
   * also the locale directory
   */
  public PkgList(String pkgroot) {
    pkgutil = new PkgUtils(pkgroot);
    model = new DefaultListModel();
    File[] fdirs = pkgutil.getPkgs();
    for(int k=0;k<fdirs.length;k++) {
      if (fdirs[k].isDirectory() &&
	  !fdirs[k].isHidden() &&
	  !fdirs[k].getName().equals("locale")) {
	model.addElement(fdirs[k].getName());
      }
    }
    setModel(model);
  }

  public boolean exists(String pkg) {
    return pkgutil.exists(pkg);
  }

  public String infoTable(String pkg) {
    return pkgutil.infoTable(pkg);
  }

  public String dependencyTable(String pkg) {
    return pkgutil.dependencyTable(pkg);
  }

  public Hashtable dependHash(String pkg) {
    return pkgutil.dependHash(pkg);
  }

  public String revDeps(String pkg) {
    if (revDependencies == null) {
      return "";
    }
    Hashtable hdep = (Hashtable) revDependencies.get(pkg);
    if (hdep == null) {
      return "";
    }
    StringBuffer sb = new StringBuffer();
    sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\"><th>Depending on this package:</th></tr>\n");
    for (Enumeration e = hdep.keys() ; e.hasMoreElements() ;) {
      String pkgdep = (String) e.nextElement();
      sb.append("<tr><td>").append(pkgdep).append("</td></tr>\n");
    }
    sb.append("</table>\n");
    return sb.toString();
  }

  /**
   * Create a dependency tree. Just walk through the elements in the
   * model and get the dependHash for each one
   */
  public void createDependencies() {
    Dependencies = new Hashtable();
    for (Enumeration e = model.elements() ; e.hasMoreElements() ;) {
      String pkgname = (String) e.nextElement();
      Dependencies.put(pkgname, dependHash(pkgname));
    }
  }

  /**
   * Create a reverse dependency tree. Pull the dependencies out
   * and populate another Hashtable.
   */
  public void createRevDependencies() {
    if (Dependencies == null) {
      createDependencies();
    }
    revDependencies = new Hashtable();
    for (Enumeration e = Dependencies.keys() ; e.hasMoreElements() ;) {
      String pkgname = (String) e.nextElement();
      Hashtable hdep = (Hashtable) Dependencies.get(pkgname);
      for (Enumeration e2 = hdep.keys() ; e2.hasMoreElements() ;) {
	String pkgdep = (String) e2.nextElement();
	String dtype = (String) hdep.get(pkgdep);
	if (dtype.equals("P")) {
	  Hashtable hdep2 = (Hashtable) revDependencies.get(pkgdep);
	  if (hdep2 == null) {
	    hdep2 = new Hashtable();
	    revDependencies.put(pkgdep, hdep2);
	  }
	  hdep2.put(pkgname, pkgname);
	}
      }
    }
  }
}
