/**
 * PkgUtils - shows available Solaris packages
 * @author Peter Tribble
 * @version 1.0
 *
 */

package uk.co.petertribble.solview;

import java.util.*;
import java.io.*;
import uk.co.petertribble.jumble.*;

public class PkgUtils {

  private File pkgrootf;
  private File[] fdirs;

  /**
   * Make a package list
   */
  public PkgUtils() {
    this("/var/sadm/pkg");
  }

  /**
   * Make a package list
   */
  public PkgUtils(String pkgroot) {
    pkgrootf = new File(pkgroot);
    fdirs = pkgrootf.listFiles();
    Arrays.sort(fdirs);
  }

  public File[] getPkgs() {
    return fdirs;
  }

  /**
   * Parse the depend file
   */
  public Hashtable dependHash(String pkg) {
    Hashtable h = new Hashtable();
    StringTokenizer st = new StringTokenizer(getDepend(pkg), "\n");
    while (st.hasMoreTokens()) {
      String[] ds = st.nextToken().split("\\s+",3);
      //Must check we got at least 2 back
      if ((ds[0].equals("P")) || (ds[0].equals("I")) || (ds[0].equals("R"))) {
	h.put(ds[1],ds[0]);
      }
    }
    return h;
  }

  /**
   * Returns the Dependency list as a html table
   */
  public String dependencyTable(String pkg) {
    StringBuffer sb = new StringBuffer();
    Hashtable h = dependHash(pkg);
    sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\"><th>Package</th><th>Dependency</th></tr>\n");
    Enumeration e1 = h.keys();
    while (e1.hasMoreElements()) {
      String s = (String) e1.nextElement();
      String d = (String) h.get(s);
      sb.append("<tr><td>").append(s).append("</td><td>");
      if (d.equals("P")) {
	sb.append("Prerequisite");
      } else if (d.equals("I")) {
	sb.append("Incompatible");
      } else if (d.equals("R")) {
	sb.append("Requisite for");
      } else {
	sb.append("Unknown");
      }
      sb.append("</td></tr>\n");
    }
    sb.append("</table>\n");
    return sb.toString();
  }

  /**
   * Returns the pkginfo file as a html table
   */
  public String infoTable(String pkg) {
    StringBuffer sb = new StringBuffer();
    Hashtable h = JumbleUtils.stringToPropHash(getInfo(pkg),"\n");
    //clean out the junk
    h.remove("PKG");
    h.remove("MAXINST");
    h.remove("PKGINST");
    h.remove("PKGSAV");
    h.remove("OAMBASE");
    h.remove("ARCH");
    h.remove("TZ");
    h.remove("LANG");
    h.remove("SUNW_PKGVERS");
    sb.append("<table width=\"100%\"><tr bgcolor=\"#eeeeee\"><th>Property</th><th>Value</th></tr>\n");
    Enumeration e1 = h.keys();
    while (e1.hasMoreElements()) {
      String s = (String) e1.nextElement();
      sb.append("<tr><td>").append(s).append("</td><td>");
      sb.append((String) h.get(s)).append("</td></tr>\n");
    }
    sb.append("</table>\n");
    return sb.toString();    
  }

  /**
   * Returns whether this package exists, by seeing whether the
   * directory corresponding to its name exists.
   */
  public boolean exists(String pkg) {
    File f = new File(pkgrootf,pkg);
    return f.exists();
  }

  /**
   * Returns the pkginfo file associated with this package as a String
   */
  public String getInfo(String pkg) {
    return JumbleFile.getStringContents(new File(pkgrootf,pkg+"/pkginfo"));
  }

  /**
   * Returns the depend file associated with this package as a String
   */
  public String getDepend(String pkg) {
    return JumbleFile.getStringContents(new File(pkgrootf,pkg+"/install/depend"));
  }

}
